// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

library detester.core;

import 'package:darmatch/matchers.dart';

part 'src/core/exceptions.dart';
part 'src/core/suite.dart';
part 'src/core/runner.dart';
part 'src/core/reporter.dart';

/// [Function] that takes no parameter. It can be used
/// as a [Test] -- see [FunctionTest].
typedef NullaryFunction();

/// Evaluates the [expectation] against the [value] and if it doesn't match,
/// throws [ExpectationFailedException] with description of the mismatch.
/// If [message] is provided, it is prepended to the exception message.
void expectThat(value, Matcher expectation, [String message]) {
  bool match = expectation.matches(value);
  if (!match) {
    var exceptionMessage = "";
    if (message != null) {
      exceptionMessage = "$message\n";
    }
    if (expectation is HasRichMessage) {
      exceptionMessage += expectation.buildMessage(value);
    } else {
      exceptionMessage += "'$value' must ${expectation.description}";
    }
    throw new ExpectationFailedException(exceptionMessage);
  }
}

/// Evaluates the [assumption] against the [value] and if it doesn't match,
/// throws [AssumptionFailedException] with description of the mismatch.
/// If [message] is provided, it is prepended to the exception message.
void assumeThat(value, Matcher assumption, [String message]) {
  bool match = assumption.matches(value);
  if (!match) {
    var exceptionMessage = "";
    if (message != null) {
      exceptionMessage = "$message\n";
    }
    if (assumption is HasRichMessage) {
      exceptionMessage += assumption.buildMessage(value);
    } else {
      exceptionMessage += "'$value' was assumed to ${assumption.description}";
    }
    throw new AssumptionFailedException("$exceptionMessage");
  }
}
