// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of detester.console;

class ConsoleReporter implements TestReporter {
  int counter = 0;
  int total = 0;

  int passed = 0;
  int failed = 0;
  int skipped = 0;
  int errors = 0;

  int fatals = 0;

  start(TestSuite suite) {
    total += suite.tests.length; // start can be called many times!

    print(suite.name);
    var dashes = new StringBuffer();
    for (int i = 0; i < suite.name.length; i++) {
      dashes.write('─');
    }
    print("$dashes");
  }

  _testName(Test test) => test.name != null ? test.name : "<test no. $counter>";

  _exceptionDescription(e, StackTrace stackTrace) =>
      "$e"
      "\n\n"
      "Exception type:\n"
      "---------------\n"
      "${e.runtimeType}"
      "\n\n"
      "Stack trace:\n"
      "------------\n"
      "${stackTrace.stackTrace}".trim().replaceAll("\n", "\n  ");

  begin(Test test) {
    counter++;
    stdout.write(white('> ${_testName(test)}'));
  }

  pass(Test test) {
    passed++;
    stdout.write(lineBegin);
    print(lightGreen("✔ ${_testName(test)}"));
  }

  fail(Test test, ExpectationFailedException failedExpectation, StackTrace stackTrace) {
    failed++;
    stdout.write(lineBegin);
    print(lightRed("✘ ${_testName(test)}"));
    print(gray("  ${_exceptionDescription(failedExpectation, stackTrace)}"));
  }

  skip(Test test, AssumptionFailedException failedAssumption, StackTrace stackTrace) {
    skipped++;
    stdout.write(lineBegin);
    print(lightBlue("∅ ${_testName(test)}"));
    print(gray("  ${_exceptionDescription(failedAssumption, stackTrace)}"));
  }

  error(Test test, exception, StackTrace stackTrace) {
    errors++;
    stdout.write(lineBegin);
    print(yellow("‼ ${_testName(test)}"));
    print(gray("  ${_exceptionDescription(exception, stackTrace)}"));
  }

  fatal(exception, StackTrace stackTrace) {
    fatals++;
    stdout.write(lineBegin);
    print(lightMagenta("✞ ${_exceptionDescription(exception, stackTrace)}"));
  }

  end() {
    count(int n) => n == 1 ? "1 test" : "$n tests";

    var fatalsDesc = fatals == 0 ? "" : ", $fatals fatal error${fatals > 1 ? "s" : ""}";

    print("");
    print("${count(counter)} out of total ${count(total)}$fatalsDesc");
    print("${count(passed)} passed, ${count(failed)} failed, ${count(skipped)} skipped, ${count(errors)} in error");
  }
}
