// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of detester.core;

abstract class TestRunner {
  run(TestSuite suite);
}

class DefaultRunner implements TestRunner {
  final TestReporter reporter;
  bool _shouldStop = false;

  DefaultRunner._(this.reporter);

  factory DefaultRunner(TestReporter reporter) {
    if (reporter == null) {
      reporter = new NullReporter();
    }
    return new DefaultRunner._(reporter);
  }

  run(TestSuite suite) {
    reporter.start(suite);

    if (suite.beforeSuite != null) {
      try {
        suite.beforeSuite();
      } catch (e, s) {
        reporter.fatal(e, s);
        _shouldStop = true;
      }
    }

    try {
      for (var test in suite.tests) {
        if (_shouldStop) {
          break;
        }
        _runTest(suite, test);
      }
    } catch (e, s) {
      reporter.fatal(e, s);
    }

    if (suite.afterSuite != null) {
      try {
        suite.afterSuite();
      } catch (e, s) {
        reporter.fatal(e, s);
      }
    }

    reporter.end();
  }

  _runTest(TestSuite suite, Test test) {
    if (suite.before != null) {
      suite.before();
    }

    try {
      reporter.begin(test);
      test();
      reporter.pass(test);
    } on ExpectationFailedException catch (e, s) {
      reporter.fail(test, e, s);
    } on AssumptionFailedException catch (e, s) {
      reporter.skip(test, e, s);
    } catch (e, s) {
      reporter.error(test, e, s);
      _shouldStop = true;
    }

    if (suite.after != null) {
      suite.after();
    }
  }
}
