// Copyright (c) 2012, Ladislav Thon. All rights reserved. Use of this source
// code is governed by a BSD-style license that can be found in the LICENSE file.

part of detester.core;

abstract class Test {
  String get name;

  call();
}

class FunctionTest implements Test {
  static final _FUNCTION_NAME_PATTERN = new RegExp("from Function '(.*?)':");

  final String name;
  final NullaryFunction _function;

  FunctionTest._(this.name, this._function);

  factory FunctionTest(NullaryFunction function) {
    var functionNameMatch = _FUNCTION_NAME_PATTERN.firstMatch("$function");
    var functionName = functionNameMatch != null ? functionNameMatch[1] : null;
    return new FunctionTest._(functionName, function);
  }

  call() => _function();
}

class TestSuite {
  final String name;
  final List<Test> tests;
  final NullaryFunction before, after, beforeSuite, afterSuite;

  TestSuite._(this.name, this.tests, this.before, this.after,
              this.beforeSuite, this.afterSuite);

  /// The [tests] list can either contain [Test]s or [NullaryFunction]s.
  /// Functions will be converted to [FunctionTest]s.
  factory TestSuite(String name, List tests, {NullaryFunction before, NullaryFunction after,
                    NullaryFunction beforeSuite, NullaryFunction afterSuite}) {

    List<Test> realTests = <Test>[];
    for (var test in tests) {
      if (test is Test) {
        realTests.add(test);
      } else if (test is NullaryFunction) {
        realTests.add(new FunctionTest(test));
      } else {
        throw new DetesterException("Test must either be a 'Test' or"
            " a 'NullaryFunction', but got '${test.runtimeType}': $test");
      }
    }

    return new TestSuite._(name, realTests, before, after, beforeSuite, afterSuite);
  }
}
